@@ -1,6 +1,6 @@
 
 /*
- * $Id: structs.h,v 1.408.2.24 2004/04/18 23:43:30 hno Exp $
+ * $Id: structs.h,v 1.506 2006/11/29 15:58:52 adrian Exp $
  *
  *
  * SQUID Web Proxy Cache          http://www.squid-cache.org/
@@ -48,6 +48,14 @@ struct _dlink_list {
     dlink_node *tail;
 };
 
+#if USE_SSL
+struct _acl_cert_data {
+    splayNode *values;
+    char *attribute;
+};
+
+#endif
+
 struct _acl_user_data {
     splayNode *names;
     struct {
@@ -57,7 +65,7 @@ struct _acl_user_data {
 };
 
 struct _acl_user_ip_data {
-    size_t max;
+    int max;
     struct {
 	unsigned int strict:1;
     } flags;
@@ -88,6 +96,13 @@ struct _acl_proxy_auth_match_cache {
     void *acl_data;
 };
 
+struct _acl_hdr_data {
+    acl_hdr_data *next;
+    relist *reglist;
+    http_hdr_type hdr_id;
+    const char *hdr_name;
+};
+
 struct _auth_user_hash_pointer {
     /* first two items must be same as hash_link */
     char *key;
@@ -111,15 +126,14 @@ struct _auth_user_t {
     int auth_module;
     /* we only have one username associated with a given auth_user struct */
     auth_user_hash_pointer *usernamehash;
-    /* we may have many proxy-authenticate strings that decode to the same user */
-    dlink_list proxy_auth_list;
+    /* cache of acl lookups on this username */
     dlink_list proxy_match_cache;
     /* what ip addresses has this user been seen at?, plus a list length cache */
     dlink_list ip_list;
-    size_t ipcount;
+    int ipcount;
     long expiretime;
     /* how many references are outstanding to this instance */
-    size_t references;
+    int references;
     /* the auth scheme has it's own private data area */
     void *scheme_data;
     /* the auth_user_request structures that link to this. Yes it could be a splaytree
@@ -137,7 +151,7 @@ struct _auth_user_request_t {
     /* any scheme specific request related data */
     void *scheme_data;
     /* how many 'processes' are working on this data */
-    size_t references;
+    int references;
     /* We only attempt authentication once per http request. This 
      * is to allow multiple auth acl references from different _access areas
      * when using connection based authentication
@@ -168,6 +182,7 @@ struct _authscheme_entry {
     AUTHSDECODE *decodeauth;
     AUTHSDIRECTION *getdirection;
     AUTHSPARSE *parse;
+    AUTHSCHECKCONFIG *checkconfig;
     AUTHSINIT *init;
     AUTHSREQFREE *requestFree;
     AUTHSSHUTDOWN *donefunc;
@@ -214,12 +229,15 @@ struct _String {
 struct _header_mangler {
     acl_access *access_list;
     char *replacement;
+    /* What follows is only used by HDR_OTHER to build a list of named headers */
+    char *name;
+    header_mangler *next;
 };
 
 struct _body_size {
     dlink_node node;
     acl_access *access_list;
-    size_t maxsize;
+    squid_off_t maxsize;
 };
 
 struct _http_version_t {
@@ -240,6 +258,7 @@ struct _snmp_request_t {
     struct snmp_pdu *PDU;
     aclCheck_t *acl_checklist;
     u_char *community;
+    struct snmp_session session;
 };
 
 #endif
@@ -326,20 +345,50 @@ struct _relist {
     relist *next;
 };
 
+struct _acl_request_type {
+    unsigned int accelerated:1;
+    unsigned int transparent:1;
+    unsigned int internal:1;
+};
+
 struct _sockaddr_in_list {
     struct sockaddr_in s;
     sockaddr_in_list *next;
 };
 
+struct _http_port_list {
+    http_port_list *next;
+    struct sockaddr_in s;
+    char *protocol;		/* protocol name */
+    char *name;			/* visible name */
+    char *defaultsite;		/* default web site */
+    char *urlgroup;		/* default urlgroup */
+    unsigned int transparent;	/* transparent proxy */
+    unsigned int accel;		/* HTTP accelerator */
+    unsigned int vhost;		/* uses host header */
+    unsigned int vport;		/* virtual port support */
+    unsigned int no_connection_auth;	/* Don't support connection oriented auth */
+#if LINUX_TPROXY
+    unsigned int tproxy;
+#endif
+};
+
 #if USE_SSL
 struct _https_port_list {
-    https_port_list *next;
-    struct sockaddr_in s;
+    http_port_list http;	/* must be first */
     char *cert;
     char *key;
     int version;
     char *cipher;
     char *options;
+    char *clientca;
+    char *cafile;
+    char *capath;
+    char *crlfile;
+    char *dhfile;
+    char *sslflags;
+    char *sslcontext;
+    SSL_CTX *sslContext;
 };
 
 #endif
@@ -376,21 +425,22 @@ struct _RemovalPolicySettings {
 
 struct _SquidConfig {
     struct {
-	size_t maxSize;
+	squid_off_t maxSize;
 	int highWaterMark;
 	int lowWaterMark;
     } Swap;
-    size_t memMaxSize;
+    squid_off_t memMaxSize;
     struct {
 	char *relayHost;
 	u_short relayPort;
 	peer *peer;
     } Wais;
     struct {
-	size_t min;
+	squid_off_t min;
 	int pct;
-	size_t max;
+	squid_off_t max;
     } quickAbort;
+    squid_off_t readAheadGap;
     RemovalPolicySettings *replPolicy;
     RemovalPolicySettings *memPolicy;
     time_t negativeTtl;
@@ -419,8 +469,9 @@ struct _SquidConfig {
 	time_t idns_query;
 #endif
     } Timeout;
-    size_t maxRequestHeaderSize;
-    size_t maxRequestBodySize;
+    squid_off_t maxRequestHeaderSize;
+    squid_off_t maxRequestBodySize;
+    squid_off_t maxReplyHeaderSize;
     dlink_list ReplyBodySize;
     struct {
 	u_short icp;
@@ -432,7 +483,7 @@ struct _SquidConfig {
 #endif
     } Port;
     struct {
-	sockaddr_in_list *http;
+	http_port_list *http;
 #if USE_SSL
 	https_port_list *https;
 #endif
@@ -446,15 +497,25 @@ struct _SquidConfig {
 #if USE_WCCP
     struct {
 	struct in_addr router;
-	struct in_addr incoming;
-	struct in_addr outgoing;
+	struct in_addr address;
 	int version;
     } Wccp;
+#endif
+#if USE_WCCPv2
+    struct {
+	sockaddr_in_list *router;
+	struct in_addr address;
+	int forwarding_method;
+	int return_method;
+	int assignment_method;
+	int weight;
+	int rebuildwait;
+	void *info;
+    } Wccp2;
 #endif
     char *as_whois_server;
     struct {
 	char *log;
-	char *access;
 	char *store;
 	char *swap;
 #if USE_USERAGENT_LOG
@@ -466,16 +527,30 @@ struct _SquidConfig {
 #if WIP_FWD_LOG
 	char *forward;
 #endif
+	logformat *logformats;
+	customlog *accesslogs;
 	int rotateNumber;
     } Log;
     char *adminEmail;
+    char *EmailFrom;
+    char *EmailProgram;
     char *effectiveUser;
+    char *visible_appname_string;
     char *effectiveGroup;
     struct {
 #if USE_DNSSERVERS
 	char *dnsserver;
 #endif
-	wordlist *redirect;
+	struct {
+	    wordlist *command;
+	    int children;
+	    int concurrency;
+	} url_rewrite;
+	struct {
+	    wordlist *command;
+	    int children;
+	    int concurrency;
+	} location_rewrite;
 #if USE_ICMP
 	char *pinger;
 #endif
@@ -483,21 +558,18 @@ struct _SquidConfig {
 	char *unlinkd;
 #endif
 	char *diskd;
+#if USE_SSL
+	char *ssl_password;
+#endif
     } Program;
 #if USE_DNSSERVERS
     int dnsChildren;
 #endif
-    int redirectChildren;
     time_t authenticateGCInterval;
     time_t authenticateTTL;
     time_t authenticateIpTTL;
-    struct {
-	int single_host;
-	char *host;
-	u_short port;
-    } Accel;
     char *appendDomain;
-    size_t appendDomainLen;
+    int appendDomainLen;
     char *debugOptions;
     char *pidFilename;
     char *mimeTablePathname;
@@ -521,8 +593,8 @@ struct _SquidConfig {
 #endif
 	struct in_addr client_netmask;
     } Addrs;
-    size_t tcpRcvBufsz;
-    size_t udpMaxHitObjsz;
+    squid_off_t tcpRcvBufsz;
+    squid_off_t udpMaxHitObjsz;
     wordlist *hierarchy_stoplist;
     wordlist *mcast_group_list;
     wordlist *dns_testname_list;
@@ -542,10 +614,10 @@ struct _SquidConfig {
     cachemgr_passwd *passwd_list;
     struct {
 	int objectsPerBucket;
-	size_t avgObjectSize;
-	size_t maxObjectSize;
-	size_t minObjectSize;
-	size_t maxInMemObjSize;
+	squid_off_t avgObjectSize;
+	squid_off_t maxObjectSize;
+	squid_off_t minObjectSize;
+	squid_off_t maxInMemObjSize;
     } Store;
     struct {
 	int high;
@@ -554,9 +626,7 @@ struct _SquidConfig {
     } Netdb;
     struct {
 	int log_udp;
-#if USE_DNSSERVERS
 	int res_defnames;
-#endif
 	int anonymizer;
 	int client_db;
 	int query_icmp;
@@ -569,7 +639,6 @@ struct _SquidConfig {
 	int log_mime_hdrs;
 	int log_fqdn;
 	int announce;
-	int accel_with_proxy;
 	int mem_pools;
 	int test_reachability;
 	int half_closed_clients;
@@ -585,6 +654,7 @@ struct _SquidConfig {
 	int ignore_unknown_nameservers;
 	int client_pconns;
 	int server_pconns;
+	int error_pconns;
 #if USE_CACHE_DIGESTS
 	int digest_generation;
 #endif
@@ -594,16 +664,33 @@ struct _SquidConfig {
 	int pipeline_prefetch;
 	int request_entities;
 	int detect_broken_server_pconns;
+	int balance_on_multiple_ip;
+	int collapsed_forwarding;
+	int relaxed_header_parser;
+	int accel_no_pmtu_disc;
+	int global_internal_static;
+	int httpd_suppress_version_string;
+	int via;
+	int check_hostnames;
+	int allow_underscore;
+	int cache_vary;
+#if FOLLOW_X_FORWARDED_FOR
+	int acl_uses_indirect_client;
+	int delay_pool_uses_indirect_client;
+	int log_uses_indirect_client;
+#endif
     } onoff;
     acl *aclList;
     struct {
 	acl_access *http;
+	acl_access *http2;
 	acl_access *icp;
 	acl_access *miss;
 	acl_access *NeverDirect;
 	acl_access *AlwaysDirect;
 	acl_access *ASlists;
 	acl_access *noCache;
+	acl_access *log;
 #if SQUID_SNMP
 	acl_access *snmp;
 #endif
@@ -611,10 +698,19 @@ struct _SquidConfig {
 #if USE_IDENT
 	acl_access *identLookup;
 #endif
-	acl_access *redirector;
+	acl_access *url_rewrite;
+	acl_access *location_rewrite;
 	acl_access *reply;
 	acl_address *outgoing_address;
 	acl_tos *outgoing_tos;
+#if USE_HTCP
+	acl_access *htcp;
+	acl_access *htcp_clr;
+#endif
+#if FOLLOW_X_FORWARDED_FOR
+	acl_access *followXFF;
+#endif
+	acl_access *vary_encoding;
     } accessList;
     acl_deny_info_list *denyInfoList;
     struct _authConfig {
@@ -623,7 +719,7 @@ struct _SquidConfig {
 	int n_configured;
     } authConfig;
     struct {
-	size_t list_width;
+	int list_width;
 	int list_wrap;
 	char *anon_user;
 	int passive;
@@ -643,9 +739,10 @@ struct _SquidConfig {
     char *errorDirectory;
     struct {
 	int maxtries;
+	int onerror;
     } retry;
     struct {
-	size_t limit;
+	squid_off_t limit;
     } MemPools;
 #if DELAY_POOLS
     delayConfig Delay;
@@ -660,7 +757,7 @@ struct _SquidConfig {
     } comm_incoming;
     int max_open_disk_fds;
     int uri_whitespace;
-    size_t rangeOffsetLimit;
+    squid_off_t rangeOffsetLimit;
 #if MULTICAST_MISS_STREAM
     struct {
 	struct in_addr addr;
@@ -677,31 +774,45 @@ struct _SquidConfig {
 	int bits_per_entry;
 	time_t rebuild_period;
 	time_t rewrite_period;
-	size_t swapout_chunk_size;
+	squid_off_t swapout_chunk_size;
 	int rebuild_chunk_percentage;
     } digest;
 #endif
 #if USE_SSL
     struct {
 	int unclean_shutdown;
+	char *ssl_engine;
     } SSL;
 #endif
-    wordlist *ext_methods;
     struct {
 	int high_rptm;
 	int high_pf;
-	size_t high_memory;
+	squid_off_t high_memory;
     } warnings;
     char *store_dir_select_algorithm;
     int sleep_after_fork;	/* microseconds */
+    time_t minimum_expiry_time;	/* seconds */
     external_acl *externalAclHelperList;
+    errormap *errorMapList;
+#if USE_SSL
+    struct {
+	char *cert;
+	char *key;
+	int version;
+	char *options;
+	char *cipher;
+	char *cafile;
+	char *capath;
+	char *crlfile;
+	char *flags;
+	SSL_CTX *sslContext;
+    } ssl_client;
+#endif
+    time_t refresh_stale_window;
+    int umask;
 };
 
 struct _SquidConfig2 {
-    struct {
-	char *prefix;
-	int on;
-    } Accel;
     struct {
 	int enable_purge;
     } onoff;
@@ -717,35 +828,33 @@ struct _close_handler {
 
 struct _dread_ctrl {
     int fd;
-    off_t offset;
-    int req_len;
+    off_t file_offset;
+    size_t req_len;
     char *buf;
     int end_of_file;
     DRCB *handler;
     void *client_data;
 };
 
-struct _dnsserver_t {
-    int id;
-    int inpipe;
-    int outpipe;
-    time_t answer;
-    off_t offset;
-    size_t size;
-    char ip_inbuf[DNS_INBUF_SZ];
-    struct timeval dispatch_time;
-    void *data;
-};
-
 struct _dwrite_q {
     off_t file_offset;
     char *buf;
-    int len;
-    off_t buf_offset;
+    size_t len;
+    size_t buf_offset;
     dwrite_q *next;
     FREE *free_func;
 };
 
+struct _CommWriteStateData {
+    int valid;
+    char *buf;
+    size_t size;
+    size_t offset;
+    CWCB *handler;
+    void *handler_data;
+    FREE *free_func;
+};
+
 
 /* ETag support is rudimantal;
  * this struct is likely to change
@@ -776,10 +885,12 @@ struct _fde {
 	unsigned int called_connect:1;
 	unsigned int nodelay:1;
 	unsigned int close_on_exec:1;
-	unsigned int read_pending:1;
+	unsigned int backoff:1;	/* keep track of whether the fd is backed off */
     } flags;
-    int bytes_read;
-    int bytes_written;
+    comm_pending read_pending;
+    comm_pending write_pending;
+    squid_off_t bytes_read;
+    squid_off_t bytes_written;
     int uses;			/* ie # req's over persistent conn */
     struct _fde_disk {
 	DWCB *wrt_handle;
@@ -799,12 +910,19 @@ struct _fde {
     close_handler *close_handler;	/* linked list */
     DEFER *defer_check;		/* check if we should defer read */
     void *defer_data;
-    CommWriteStateData *rwstate;	/* State data for comm_write */
+    struct _CommWriteStateData rwstate;		/* State data for comm_write */
     READ_HANDLER *read_method;
     WRITE_HANDLER *write_method;
 #if USE_SSL
     SSL *ssl;
-    int ssl_shutdown:1;
+#endif
+#ifdef _SQUID_MSWIN_
+    struct {
+	long handle;
+    } win32;
+#endif
+#if DELAY_POOLS
+    int slow_id;
 #endif
 };
 
@@ -826,15 +944,15 @@ struct _MemBuf {
     /* private, stay away; use interface function instead */
     mb_size_t max_capacity;	/* when grows: assert(new_capacity <= max_capacity) */
     mb_size_t capacity;		/* allocated space */
-    FREE *freefunc;		/* what to use to free the buffer, NULL after memBufFreeFunc() is called */
+    unsigned stolen:1;		/* the buffer has been stolen for use by someone else */
 };
 
 /* see Packer.c for description */
 struct _Packer {
     /* protected, use interface functions instead */
     append_f append;
-    vprintf_f vprintf;
-    void *real_handler;		/* first parameter to real append and vprintf */
+    vprintf_f packer_vprintf;
+    void *real_handle;		/* first parameter to real append and vprintf */
 };
 
 /* http status line */
@@ -866,12 +984,13 @@ struct _HttpHdrCc {
     int max_age;
     int s_maxage;
     int max_stale;
+    String other;
 };
 
 /* http byte-range-spec */
 struct _HttpHdrRangeSpec {
-    ssize_t offset;
-    ssize_t length;
+    squid_off_t offset;
+    squid_off_t length;
 };
 
 /* There may be more than one byte range specified in the request.
@@ -885,12 +1004,12 @@ struct _HttpHdrRange {
 /* http content-range header field */
 struct _HttpHdrContRange {
     HttpHdrRangeSpec spec;
-    ssize_t elength;		/* entity length, not content length */
+    squid_off_t elength;	/* entity length, not content length */
 };
 
 /* some fields can hold either time or etag specs (e.g. If-Range) */
 struct _TimeOrTag {
-    ETag tag;			/* entity tag */
+    const char *tag;		/* entity tag */
     time_t time;
     int valid;			/* true if struct is usable */
 };
@@ -899,8 +1018,8 @@ struct _TimeOrTag {
 struct _HttpHdrRangeIter {
     HttpHdrRangePos pos;
     const HttpHdrRangeSpec *spec;	/* current spec at pos */
-    ssize_t debt_size;		/* bytes left to send from the current spec */
-    ssize_t prefix_size;	/* the size of the incoming HTTP msg prefix */
+    squid_off_t debt_size;	/* bytes left to send from the current spec */
+    squid_off_t prefix_size;	/* the size of the incoming HTTP msg prefix */
     String boundary;		/* boundary for multipart responses */
 };
 
@@ -947,7 +1066,7 @@ struct _HttpReply {
     int hdr_sz;			/* sums _stored_ status-line, headers, and <CRLF> */
 
     /* public, readable; never update these or their .hdr equivalents directly */
-    int content_length;
+    squid_off_t content_length;
     time_t date;
     time_t last_modified;
     time_t expires;
@@ -971,13 +1090,15 @@ struct _http_state_flags {
     unsigned int only_if_cached:1;
     unsigned int keepalive_broken:1;
     unsigned int abuse_detected:1;
+    unsigned int request_sent:1;
+    unsigned int front_end_https:2;
+    unsigned int originpeer:1;
 };
 
 struct _HttpStateData {
     StoreEntry *entry;
     request_t *request;
-    char *reply_hdr;
-    size_t reply_hdr_size;
+    MemBuf reply_hdr;
     int reply_hdr_state;
     peer *peer;			/* peer request made to */
     int eof;			/* reached end-of-object? */
@@ -1038,11 +1159,14 @@ struct _AccessLogEntry {
     } icp;
     struct {
 	struct in_addr caddr;
-	size_t size;
+	squid_off_t size;
 	log_type code;
 	int msec;
 	const char *rfc931;
 	const char *authuser;
+#if USE_SSL
+	const char *ssluser;
+#endif
     } cache;
     struct {
 	char *request;
@@ -1052,19 +1176,29 @@ struct _AccessLogEntry {
 	const char *method_str;
     } private;
     HierarchyLogEntry hier;
+    HttpReply *reply;
+    request_t *request;
 };
 
 struct _clientHttpRequest {
     ConnStateData *conn;
+    dlink_node node;
     request_t *request;		/* Parsed URL ... */
+    request_t *orig_request;	/* Parsed URL ... */
     store_client *sc;		/* The store_client we're using */
     store_client *old_sc;	/* ... for entry to be validated */
     char *uri;
     char *log_uri;
     struct {
-	off_t offset;
-	size_t size;
+	squid_off_t offset;
+	squid_off_t size;
     } out;
+    HttpReply *reply;		/* it is important for clientHttpRequest
+				 * to have its own HttpReply for
+				 * logging, especially in cases
+				 * where the reply headers sent to
+				 * the client are different than
+				 * those received from the origin server. */
     HttpHdrRangeIter range_iter;	/* data for iterating thru range specs */
     size_t req_sz;		/* raw request size on input, not current request size */
     StoreEntry *entry;
@@ -1077,31 +1211,38 @@ struct _clientHttpRequest {
     http_version_t http_ver;
     int redirect_state;
     aclCheck_t *acl_checklist;	/* need ptr back so we can unreg if needed */
-    clientHttpRequest *next;
     AccessLogEntry al;
     struct {
 	unsigned int accel:1;
+	unsigned int transparent:1;
 	unsigned int internal:1;
 	unsigned int done_copying:1;
 	unsigned int purging:1;
+	unsigned int hit:1;
     } flags;
     struct {
 	http_status status;
 	char *location;
     } redirect;
     dlink_node active;
-    size_t maxBodySize;
+    squid_off_t maxBodySize;
+    /*
+     * This can be changed into a pointer later on when the memory allocator
+     * has been taught the difference between buffers to zero and buffers not to
+     * zero.. [ahc]
+     */
+    char readbuf[CLIENT_SOCK_SZ];
 };
 
 struct _ConnStateData {
     int fd;
     struct {
 	char *buf;
-	off_t offset;
+	size_t offset;
 	size_t size;
     } in;
     struct {
-	size_t size_left;	/* How much body left to process */
+	squid_off_t size_left;	/* How much body left to process */
 	request_t *request;	/* Parameters passed to clientReadBody */
 	char *buf;
 	size_t bufsize;
@@ -1113,7 +1254,7 @@ struct _ConnStateData {
     /* note this is ONLY connection based because NTLM is against HTTP spec */
     /* the user details for connection based authentication */
     auth_user_request_t *auth_user_request;
-    clientHttpRequest *chr;
+    dlink_list reqs;
     struct sockaddr_in peer;
     struct sockaddr_in me;
     struct in_addr log_addr;
@@ -1123,6 +1264,16 @@ struct _ConnStateData {
 	int n;
 	time_t until;
     } defer;
+    http_port_list *port;
+    int transparent;
+    struct {
+	int fd;			/* pinned server side connection */
+	char *host;		/* host name of pinned connection */
+	int port;		/* port of pinned connection */
+	int pinned;		/* this connection was pinned */
+	int auth;		/* pinned for www authentication */
+	peer *peer;		/* peer the connection goes via */
+    } pinning;
 };
 
 struct _ipcache_addrs {
@@ -1171,8 +1322,8 @@ struct _DigestFetchState {
     store_client *sc;
     store_client *old_sc;
     request_t *request;
-    int offset;
-    int mask_offset;
+    squid_off_t offset;
+    squid_off_t mask_offset;
     time_t start_time;
     time_t resp_time;
     time_t expires;
@@ -1226,6 +1377,7 @@ struct _PeerDigest {
 #endif
 
 struct _peer {
+    char *name;
     char *host;
     peer_t type;
     struct sockaddr_in in_addr;
@@ -1271,12 +1423,19 @@ struct _peer {
 	unsigned int closest_only:1;
 #if USE_HTCP
 	unsigned int htcp:1;
+	unsigned int htcp_oldsquid:1;
 #endif
 	unsigned int no_netdb_exchange:1;
 #if DELAY_POOLS
 	unsigned int no_delay:1;
 #endif
 	unsigned int allow_miss:1;
+	unsigned int originserver:1;
+	unsigned int userhash:1;
+	unsigned int sourcehash:1;
+#if USE_CARP
+	unsigned int carp:1;
+#endif
     } options;
     int weight;
     struct {
@@ -1305,12 +1464,49 @@ struct _peer {
     struct {
 	unsigned int hash;
 	double load_multiplier;
-	float load_factor;
+	double load_factor;	/* normalized weight value */
     } carp;
 #endif
+    struct {
+	unsigned int hash;
+	double load_multiplier;
+	double load_factor;
+    } userhash;
+    struct {
+	unsigned int hash;
+	double load_multiplier;
+	double load_factor;
+    } sourcehash;
     char *login;		/* Proxy authorization */
     time_t connect_timeout;
     int max_conn;
+    struct {
+	char *url;
+	int min, max;
+	int interval;
+	int timeout;
+	int state;
+	time_t last;
+	PeerMonitor *data;
+    } monitor;
+    char *domain;		/* Forced domain */
+#if USE_SSL
+    int use_ssl;
+    char *sslcert;
+    char *sslkey;
+    int sslversion;
+    char *ssloptions;
+    char *sslcipher;
+    char *sslcafile;
+    char *sslcapath;
+    char *sslcrlfile;
+    char *sslflags;
+    char *ssldomain;
+    SSL_CTX *sslContext;
+    SSL_SESSION *sslSession;
+#endif
+    int front_end_https;
+    int connection_auth;
 };
 
 struct _net_db_name {
@@ -1415,20 +1611,21 @@ struct _iostats {
 struct _mem_node {
     char data[SM_PAGE_SIZE];
     int len;
+    int uses;
     mem_node *next;
 };
 
 struct _mem_hdr {
     mem_node *head;
     mem_node *tail;
-    int origin_offset;
+    squid_off_t origin_offset;
 };
 
 /* keep track each client receiving data from that particular StoreEntry */
 struct _store_client {
     int type;
-    off_t copy_offset;
-    off_t seen_offset;
+    squid_off_t copy_offset;
+    squid_off_t seen_offset;
     size_t copy_size;
     char *copy_buf;
     STCB *callback;
@@ -1444,6 +1641,9 @@ struct _store_client {
     delay_id delay_id;
 #endif
     dlink_node node;
+#if STORE_CLIENT_LIST_DEBUG
+    void *owner;
+#endif
 };
 
 
@@ -1486,12 +1686,14 @@ struct _MemObject {
     method_t method;
     char *url;
     mem_hdr data_hdr;
-    off_t inmem_hi;
-    off_t inmem_lo;
+    squid_off_t inmem_hi;
+    squid_off_t inmem_lo;
+    int serverfd;		/* Record the server's fd if we have too much
+				 * data waiting to send to the client */
     dlink_list clients;
     int nclients;
     struct {
-	off_t queue_offset;	/* relative to in-mem data */
+	squid_off_t queue_offset;	/* relative to in-mem data */
 	mem_node *memnode;	/* which node we're currently paging out */
 	storeIOState *sio;
     } swapout;
@@ -1507,12 +1709,16 @@ struct _MemObject {
     char *log_url;
     RemovalPolicyNode repl;
     int id;
-    ssize_t object_sz;
+    squid_off_t object_sz;
     size_t swap_hdr_sz;
 #if URL_CHECKSUM_DEBUG
     unsigned int chksum;
 #endif
+    const char *vary_hdr;
     const char *vary_headers;
+    const char *vary_encoding;
+    StoreEntry *ims_entry;
+    time_t refresh_timestamp;
 };
 
 struct _StoreEntry {
@@ -1524,7 +1730,7 @@ struct _StoreEntry {
     time_t lastref;
     time_t expires;
     time_t lastmod;
-    size_t swap_file_sz;
+    squid_file_sz swap_file_sz;
     u_short refcount;
     u_short flags;
     /* END OF ON-DISK STORE_META_STD */
@@ -1544,7 +1750,7 @@ struct _SwapDir {
     int max_size;
     char *path;
     int index;			/* This entry's index into the swapDirs array */
-    ssize_t max_objsize;
+    squid_off_t max_objsize;
     RemovalPolicy *repl;
     int removals;
     int scanned;
@@ -1553,6 +1759,7 @@ struct _SwapDir {
 	unsigned int read_only:1;
     } flags;
     STINIT *init;		/* Initialise the fs */
+    STCHECKCONFIG *checkconfig;	/* Verify configuration */
     STNEWFS *newfs;		/* Create a new fs */
     STDUMP *dump;		/* Dump fs config snippet */
     STFREE *freefs;		/* Free the fs data */
@@ -1560,6 +1767,7 @@ struct _SwapDir {
     STSTATFS *statfs;		/* Dump fs statistics */
     STMAINTAINFS *maintainfs;	/* Replacement maintainence */
     STCHECKOBJ *checkobj;	/* Check if the fs will store an object */
+    STCHECKLOADAV *checkload;	/* Check if the fs is getting overloaded .. */
     /* These two are notifications */
     STREFOBJ *refobj;		/* Reference this object */
     STUNREFOBJ *unrefobj;	/* Unreference this object */
@@ -1572,6 +1780,7 @@ struct _SwapDir {
 	STOBJREAD *read;
 	STOBJWRITE *write;
 	STOBJUNLINK *unlink;
+	STOBJRECYCLE *recycle;
     } obj;
     struct {
 	STLOGOPEN *open;
@@ -1601,7 +1810,7 @@ struct _request_flags {
     unsigned int hierarchical:1;
     unsigned int loopdetect:1;
     unsigned int proxy_keepalive:1;
-    unsigned int proxying:1;
+    unsigned int proxying:1;	/* this should be killed, also in httpstateflags */
     unsigned int refresh:1;
     unsigned int redirected:1;
     unsigned int need_validation:1;
@@ -1609,9 +1818,20 @@ struct _request_flags {
     unsigned int nocache_hack:1;	/* for changing/ignoring no-cache requests */
 #endif
     unsigned int accelerated:1;
+    unsigned int transparent:1;
     unsigned int internal:1;
     unsigned int body_sent:1;
     unsigned int reset_tcp:1;
+    unsigned int must_keepalive:1;
+    unsigned int connection_auth:1;	/* Request wants connection oriented auth */
+    unsigned int connection_proxy_auth:1;	/* Request wants connection oriented auth */
+    unsigned int no_connection_auth:1;	/* Connection oriented auth can not be supported */
+    unsigned int pinned:1;	/* Request seont on a pinned connection */
+    unsigned int auth_sent:1;	/* Authentication forwarded */
+#if LINUX_TPROXY
+    unsigned int tproxy:1;
+#endif
+    unsigned int collapsed:1;	/* This request was collapsed. Don't trust the store entry to be valid */
 };
 
 struct _link_list {
@@ -1624,8 +1844,9 @@ struct _storeIOState {
     sfileno swap_filen;
     StoreEntry *e;		/* Need this so the FS layers can play god */
     mode_t mode;
-    size_t st_size;		/* do stat(2) after read open */
-    off_t offset;		/* current on-disk offset pointer */
+    squid_off_t st_size;	/* do stat(2) after read open */
+    squid_off_t offset;		/* current on-disk offset pointer */
+    squid_off_t write_offset;	/* current storeWrite offset pointer */
     STFNCB *file_callback;	/* called on delayed sfileno assignments */
     STIOCB *callback;
     void *callback_data;
@@ -1657,17 +1878,39 @@ struct _request_t {
     int imslen;
     int max_forwards;
     /* these in_addr's could probably be sockaddr_in's */
+    in_port_t client_port;
     struct in_addr client_addr;
+#if FOLLOW_X_FORWARDED_FOR
+    struct in_addr indirect_client_addr;	/* after following X-Forwarded-For */
+#endif				/* FOLLOW_X_FORWARDED_FOR */
     struct in_addr my_addr;
     unsigned short my_port;
     HttpHeader header;
-    ConnStateData *body_connection;	/* used by clientReadBody() */
-    int content_length;
+    squid_off_t content_length;
     HierarchyLogEntry hier;
     err_type err_type;
     char *peer_login;		/* Configured peer login:password */
     time_t lastmod;		/* Used on refreshes */
-    const char *vary_headers;	/* Used when varying entities are detected. Changes how the store key is calculated */
+    char *vary_hdr;		/* Used when varying entities are detected. Changes how the store key is calculated */
+    char *vary_headers;		/* Used when varying entities are detected. Changes how the store key is calculated */
+    String vary_encoding;	/* Used when varying entities are detected. Changes how the store key is calculated. */
+    VaryData *vary;
+    Array *etags;		/* possible known entity tags (Vary MISS) */
+    char *etag;			/* current entity tag, cache validation */
+    unsigned int done_etag:1;	/* We have done clientProcessETag on this, don't attempt it again */
+    char *urlgroup;		/* urlgroup, returned by redirectors */
+    char *peer_domain;		/* Configured peer forceddomain */
+    BODY_HANDLER *body_reader;
+    void *body_reader_data;
+    String extacl_log;		/* String to be used for access.log purposes */
+    const char *extacl_user;	/* User name returned by extacl lookup */
+    const char *extacl_passwd;	/* Password returned by extacl lookup */
+#if FOLLOW_X_FORWARDED_FOR
+    /* XXX a list of IP addresses would be a better data structure
+     * than this String */
+    String x_forwarded_for_iterator;
+#endif				/* FOLLOW_X_FORWARDED_FOR */
+    ConnStateData *pinned_connection;	/* If set then this request is tighly tied to the corresponding client side connetion */
 };
 
 struct _cachemgr_passwd {
@@ -1690,19 +1933,13 @@ struct _refresh_t {
 	unsigned int override_lastmod:1;
 	unsigned int reload_into_ims:1;
 	unsigned int ignore_reload:1;
+	unsigned int ignore_no_cache:1;
+	unsigned int ignore_private:1;
+	unsigned int ignore_auth:1;
 #endif
     } flags;
 };
 
-struct _CommWriteStateData {
-    char *buf;
-    size_t size;
-    off_t offset;
-    CWCB *handler;
-    void *handler_data;
-    FREE *free_func;
-};
-
 struct _ErrorState {
     err_type type;
     int page_id;
@@ -1711,8 +1948,6 @@ struct _ErrorState {
     request_t *request;
     char *url;
     int xerrno;
-    char *host;
-    u_short port;
     char *dnsserver_msg;
     time_t ttl;
     struct in_addr src_addr;
@@ -1795,6 +2030,10 @@ struct _StatCounters {
 	int query_timeouts;
 	int times_used;
     } icp;
+    struct {
+	int pkts_sent;
+	int pkts_recv;
+    } htcp;
     struct {
 	int requests;
     } unlink;
@@ -1846,11 +2085,8 @@ struct _StatCounters {
 	    int recvfroms;
 	    int sendtos;
 	} sock;
-#if HAVE_POLL
 	int polls;
-#else
 	int selects;
-#endif
     } syscalls;
     int aborted_requests;
     struct {
@@ -1888,6 +2124,26 @@ struct _tlv {
  * know the dirn .. 
  */
 struct _storeSwapLogData {
+    char op;
+    sfileno swap_filen;
+    time_t timestamp;
+    time_t lastref;
+    time_t expires;
+    time_t lastmod;
+    squid_file_sz swap_file_sz;
+    u_short refcount;
+    u_short flags;
+    unsigned char key[MD5_DIGEST_CHARS];
+};
+
+struct _storeSwapLogHeader {
+    char op;
+    int version;
+    int record_size;
+};
+
+#if SIZEOF_SQUID_FILE_SZ != SIZEOF_SIZE_T
+struct _storeSwapLogDataOld {
     char op;
     sfileno swap_filen;
     time_t timestamp;
@@ -1900,6 +2156,9 @@ struct _storeSwapLogData {
     unsigned char key[MD5_DIGEST_CHARS];
 };
 
+#endif
+
+
 /* object to track per-action memory usage (e.g. #idle objects) */
 struct _MemMeter {
     ssize_t level;		/* current level (count or volume) */
@@ -1920,8 +2179,14 @@ struct _MemPoolMeter {
 struct _MemPool {
     const char *label;
     size_t obj_size;
+#if DEBUG_MEMPOOL
+    size_t real_obj_size;	/* with alignment */
+#endif
     Stack pstack;		/* stack for free pointers */
     MemPoolMeter meter;
+#if DEBUG_MEMPOOL
+    MemPoolMeter diff_meter;
+#endif
 };
 
 struct _ClientInfo {
@@ -1940,12 +2205,13 @@ struct _ClientInfo {
 	int n_denied;
     } cutoff;
     int n_established;		/* number of current established connections */
+    time_t last_seen;
 };
 
 struct _CacheDigest {
     /* public, read-only */
     char *mask;			/* bit mask */
-    size_t mask_size;		/* mask size in bytes */
+    int mask_size;		/* mask size in bytes */
     int capacity;		/* expected maximum for .count, not a hard limit */
     int bits_per_entry;		/* number of bits allocated for each entry from capacity */
     int count;			/* number of digested entries */
@@ -1975,6 +2241,9 @@ struct _FwdState {
 	unsigned int dont_retry:1;
 	unsigned int ftp_pasv_failed:1;
     } flags;
+#if LINUX_NETFILTER
+    struct sockaddr_in src;
+#endif
 };
 
 #if USE_HTCP
@@ -1998,12 +2267,14 @@ struct _helper_request {
     char *buf;
     HLPCB *callback;
     void *data;
+    struct timeval dispatch_time;
 };
 
 struct _helper_stateful_request {
     char *buf;
     HLPSCB *callback;
     void *data;
+    struct timeval dispatch_time;
 };
 
 
@@ -2014,7 +2285,9 @@ struct _helper {
     const char *id_name;
     int n_to_start;
     int n_running;
+    int n_active;
     int ipc_type;
+    int concurrency;
     time_t last_queue_warn;
     struct {
 	int requests;
@@ -2033,7 +2306,9 @@ struct _helper_stateful {
     const char *id_name;
     int n_to_start;
     int n_running;
+    int n_active;
     int ipc_type;
+    int concurrency;
     MemPool *datapool;
     HLPSAVAIL *IsAvailable;
     HLPSRESET *Reset;
@@ -2053,23 +2328,23 @@ struct _helper_server {
     int pid;
     int rfd;
     int wfd;
-    char *buf;
-    size_t buf_sz;
-    off_t offset;
-    struct timeval dispatch_time;
-    struct timeval answer_time;
+    MemBuf wqueue;
+    char *rbuf;
+    size_t rbuf_sz;
+    int roffset;
     dlink_node link;
     helper *parent;
-    helper_request *request;
+    helper_request **requests;
     struct _helper_flags {
-	unsigned int alive:1;
-	unsigned int busy:1;
+	unsigned int writing:1;
 	unsigned int closing:1;
 	unsigned int shutdown:1;
     } flags;
     struct {
 	int uses;
+	unsigned int pending;
     } stats;
+    void *hIpc;
 };
 
 
@@ -2080,7 +2355,7 @@ struct _helper_stateful_server {
     int wfd;
     char *buf;
     size_t buf_sz;
-    off_t offset;
+    int offset;
     struct timeval dispatch_time;
     struct timeval answer_time;
     dlink_node link;
@@ -2099,6 +2374,7 @@ struct _helper_stateful_server {
 	int releases;
     } stats;
     void *data;			/* State data used by the calling routines */
+    void *hIpc;
 };
 
 /*
@@ -2165,10 +2441,34 @@ struct _Logfile {
     char path[MAXPATHLEN];
     char *buf;
     size_t bufsz;
-    off_t offset;
+    ssize_t offset;
     struct {
-	unsigned int fatal:1;
+	unsigned int fatal;
+	unsigned int syslog;
     } flags;
+    int syslog_priority;
+};
+
+struct _logformat {
+    char *name;
+    logformat_token *format;
+    logformat *next;
+};
+
+struct _customlog {
+    char *filename;
+    acl_list *aclList;
+    logformat *logFormat;
+    Logfile *logfile;
+    customlog *next;
+    enum {
+	CLF_UNKNOWN,
+	CLF_AUTO,
+	CLF_CUSTOM,
+	CLF_SQUID,
+	CLF_COMMON,
+	CLF_NONE
+    } type;
 };
 
 struct cache_dir_option {
@@ -2177,4 +2477,21 @@ struct cache_dir_option {
     void (*dump) (StoreEntry * e, const char *option, SwapDir * sd);
 };
 
+struct error_map_entry {
+    struct error_map_entry *next;
+    char *value;
+    int status;
+};
+struct _errormap {
+    errormap *next;
+    char *url;
+    struct error_map_entry *map;
+};
+
+struct _VaryData {
+    char *key;
+    char *etag;
+    Array etags;
+};
+
 #endif /* SQUID_STRUCTS_H */
